#!/usr/bin/env python

# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import platform
import sys
import os
import os.path

#
# Mandatory for openpluginlib initialisation
#
import bootstrap

#
# Import jahplayer modules
#
import jahwidgets
import openpluginlib
import openassetlib
import uuid
import jahstate
import utilities
import assets

# Widgets

import header
import viewport
import control
import reel
import footer

#
# Define guid generator class acceptable to oal
#

class uuid_generator( openassetlib.guid_generator ):
	def generate_guid( self ):
		return uuid.uuid_time( )

#
# The jahplayer main application
#

class jahplayer:
	def __init__( self, state ):
		# Store the state
		self.state = state

		# Construct the main jahplayer window
		self.handle = jahwidgets.create_ui_from_file(
                        utilities.get_absolute_path( "jahplayer.ui" ),
                        jahwidgets.widget_handle() )

		# Populate the frames
		self.viewport = viewport.widget( jahwidgets.find( "viewport", self.handle ), state )
		self.header = header.widget( jahwidgets.find( "header", self.handle ), state )
		self.control = control.widget( jahwidgets.find( "control", self.handle ), state )
		self.reel = reel.widget( jahwidgets.find( "reel", self.handle ), state )
		self.footer = footer.widget( jahwidgets.find( "footer", self.handle ), state )

                # make the splitter behave
                splitter_handle = jahwidgets.find( "splitter", self.handle )
                splitter = jahwidgets.splitter( splitter_handle )
                splitter.set_resize_mode( jahwidgets.find( "lower", self.handle ), jahwidgets.ResizeMode.KeepSize )

                # set the main widget
                self.state.app.set_main_widget( self.handle )
                
		# Show the window
		self.handle.show( True )

                # force resize
                # ALERT! pure drivel; remove as soon as I figure out what is
                # causing the "sticky" behaviour
                # appears to be related to player geometry delegate not being called
                # first time around, which can happen (at least on X11) if the GL
                # context is not valid.
                self.handle.resize( self.handle.width + 1, self.handle.height + 1 )
                self.handle.resize( self.handle.width - 1, self.handle.height - 1 )

#
# Main method
#

def main( ):
	# Turn on trace if required
	if len( sys.argv ) > 1 and sys.argv[ 1 ] == "--trace":
		sys.settrace( utilities.traceit )

	# Initialise the application
	app = jahwidgets.application( )

	# setup some styling
	if not app.setStyle( "v3" ):
		print "failed to load v3 theme"

	# Initialise the open libraries
	openassetlib.init( uuid_generator( ) )

	# All objects will share this state object
	state = jahstate.app_state( app )

	# Attempt to restore
	state.load_reels( )

	# Create the main jplayer instance
	jah = jahplayer( state )

	# Execute the application
	app.start_event_loop( )

	# Save state
	state.save_reels( )

	# Close opl
	openassetlib.uninit( )

#
# Run the main method if required
#

if __name__ == "__main__":
	main( )
